%!octave 
%      
%     EM
%       
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.6.4
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 

function [ rS, rmu, rw, gmax ] = em_base(X,S,mu,w,k)
   G_FACTOR = 1.0;
   MAX_ITER = 50 ;
   G_MAX = 10e-4 ; % максимально допустимое изменение параметров EM (критерий остановки EM)
   
   m = size(X,1); % количество учебных векторов
   n = size(X,2); % размерность пространства признаков

   iS = invS(S) ;
   dS = detS(S) ;

   g = zeros(m,k);
   gmax = [];

   for iter = 1:MAX_ITER,
	  g0 = g ;

	  % E-шаг
	  for i = 1:m,
		 x = X(i,:); 
		 l = size(x,2);
		 mix = p_mix(x,iS,dS,mu,w);
		 for j = 1:k,  
			s1 = j*l-(l-1); s2=s1+(l-1);
			g(i,j) = w(j) * p_gauss(x,iS(:,s1:s2 ),dS(j),mu(j,:)) / mix ;
		 endfor;
	  endfor;

	  % M-шаг
	  gs = sum(g) ;
	  w = gs/m ;

	  mu =[];
	  for j=1:k,
		 mm = sum(g(:,j).*X) / gs(j) ;
		 mu = [ mu ; mm ] ;
	  endfor;
	  
	  S = [];
	  for j = 1:k,
		 sm = zeros(n) ;
		 for i = 1:m,
			x = X(i,:) ;
			sm += (x-mu(j,:))'*(x-mu(j,:)) .* g(i,j) ;
		 endfor;
         sm = sm ./ ( gs(j)*G_FACTOR ) ;
		 S = [ S sm ] ;
	  endfor;

	  iS = invS(S) ;
	  dS = detS(S) ;

	  gm = max(max(abs(g-g0))) ;

	  gmax = [ iter gm ; gmax ] ;
   
	  printf("gm[%i]=%f/%f\r",iter,gm,G_MAX);

%	  if(iter<10), st=sprintf('00%i',iter); elseif(iter<100), st=sprintf('0%i',iter); else st=sprintf('%i',iter); endif
%     f=sprintf('result/points_%i_step_%s.png',k,st);
%	  t=sprintf('step=%i, error=%f',iter,gm);
%	  plotMix( X,S,mu,f,t ) ;

	  if(gm < G_MAX || isnan(gm) ), break; endif;
   endfor;

   if( isnan(gm) || (iter == MAX_ITER) ),
	  rS = [] ;
	  rmu = [] ;
	  rw = [] ;
	  return;
   endif
   
   rS = S ;
   rmu = mu ;
   rw = w ;

endfunction;



% $Id: em_base.m,v 1.1 2013/12/25 14:09:40 mechanoid Exp $
