%!octave 
%      
%     EM +
%       
%       OS : GNU/Linux 3.9.3-1-ARCH 
% COMPILER : GNU Octave, version 3.6.4
%
%   AUTHOR : Evgeny S. Borisov
%
%  Glushkov Institute of Cybernetics
%  National Academy of Sciences of Ukraine
% 
%    http://www.mechanoid.kiev.ua
%  e-mail : nn@mechanoid.kiev.ua
% 

function [ S, mu, w ] = em_seq(X)
   m = size(X,1) ; % количество точек
   n = size(X,2) ; % размерность пространства признаков

   R = 0.6 ; % коэффициент правдоподобия
   MAX_ITER = 7 ;

   m0 = round(m/3) ; % минимальная длина выборки, по которой можно восстанавливать плотность

   k = 1 ; % количество компонент смеси распределения
   w = [ 1 ] ; % начальное значение априорной вероятности компонент

   mu = mean(X) ; % начальное значение мат.ожидание
   S = cov(X);
   iS = inv(S) ;
   dS = det(S) ;

   plotMix( X,S,mu,sprintf('result/points_%i_init.png',k),'init' ) ;

   % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

   for iter = 1:MAX_ITER,

	  % расчитать плотность для всех объектов выборки
	  p=[] ;
	  for i = 1:m,
		 x = X(i,:) ; 
		 p = [ p ; p_mix(x,iS,dS,mu,w) ] ;
	  endfor
	  mp = max(p);  % максимальное значение плотности на выборке


	  % найти объекты со значением плотности меньше пороговой
	  % т.е. малым правдоподобием
	  p = p - R*mp ; 
	  p = abs(p)-p ;

	  low_p_idx = find(p) ;
	  low_p_size = size(low_p_idx,1) ;

	  printf("\ntest data\nk=%i/%i, 'bad' point count = %i of %i, bound = %.0f\n",k,MAX_ITER,low_p_size,m,m0);

	  % выделяем объекты с малым правдоподобием
	  U = X(low_p_idx,:) ;
	
      
	  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	  clf ; hold on ;
	  plot( X(:,1) , X(:,2) , '.b' ) ; 
	  plot( U(:,1) , U(:,2) , '.y' ) ; 
	  plot( mu(:,1) , mu(:,2) , '.r' ) ; 
	  f = sprintf("result/u-%i.png",k ) ;
	  t = sprintf("bound = %f, low p = %i/%i/%i", R*mp, low_p_size,m0, m ) ;
	  title(t);
	  print(f,'-dpng' ,'-S650,650') ;
	  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	  if( low_p_size < m0 ), break; endif;

	  % добавляем компоненту смеси
	  k+=1;
	  %m0 = m0/R;
	  R*=R;

	  % расчитываем новые матрицы ковариаций
	  cU = cov(U) ;

	  % расчитываем новые априорные вероятности компонент смеси
	  wk = size(U,1)/m ;

	  % расчитываем новые значения мат.ожидания
	  muU = mean(U)*wk ;
	  
	  printf("\nadd new mix component, start EM...\n",k,MAX_ITER,low_p_size,m0,m);

	  % выполняем EM для всей выборки с новыми параметрами 
	  [ rS, rmu, rw, gmax ] = em_base(X,[ S cU ],[ mu ; muU ] ,[ w.*(1-wk) wk ],k);

	  if( size(rS,1) != n ), % error
		 printf('\nEM fails, rollback parameters\n');
	  	return;
	  endif

      S = rS ;
	  iS = invS(S) ;
	  dS = detS(S) ;
	  mu = rmu;
	  w = rw;

	  % промежуточный результат
	  clf ; 
	  plot(gmax(:,1),gmax(:,2),'-r') ; 
	  mingmax = min(gmax(:,2)) ;
	  title( sprintf("k = %i, iter = %i, min error = %f", k, size(gmax,1), mingmax ) );
	  print( sprintf('result/gmax_%i.png',k),'-dpng' ,'-S650,650') ;
	  t = sprintf("k = %i, iter = %i, low p count = %i \n min error = %f", k, size(gmax,1),low_p_size, mingmax ) ;
	  plotMix( X,S,mu,sprintf('result/points_%i_res.png',k),t ) ;

   endfor;

endfunction


% $Id: em_seq.m,v 1.1 2013/12/25 14:09:40 mechanoid Exp $
